-- 使用 entity mesh 时，不需要 myShader
-- 使用 pure mesh 时，需要 myShader 计算光照
-- 简单定义形式必须严格使用 shader([[...]], [[...]])，不能使用 shader{vertex=[[...]], fragment=[[...]]}}

---[=[

VisualColorWorkGroup = shader.compute(
[[
#version 430
#include <codea/common.glsl>

uniform uint instanceCount;
uniform uint ulx;
uniform uint factor;
uniform vec3 colR;

// 
struct Instance
{
    vec4 position;
};

/*
layout(std430, binding = BUFFER0 ) buffer  Pos
{
    Instance instPos[ ]; 
}; 
*/
// 定义 RW 型的 buffer，变量名为 instances，单元类型为结构体 Instance，使用 BUFFER0 缓存位置 
BUFFER_RW(instPos, Instance, BUFFER0);

struct Color
{
vec4 color0;
};
BUFFER_RW(instCol, Color, BUFFER1);


//输入布局限定符声明一个 16x8x1 的本地工作组----gl_WorkGroupSize
//layout( local_size_x = 1, local_size_y = 1, local_size_z = 1 ) in;
//NUM_THREADS(1, 1, 1)
NUM_THREADS(2, 2, 2)

void main()     
{

    
    // 内置变量，只能在 compute shader 中使用
    // 本地工作组尺寸：每个维度上有多少个工作项 (local_size_x, local_size_y, local_size_z)
    vec3 lwSize = gl_WorkGroupSize;
    
    // 全局工作组尺寸：每个全局工作组中有多少个本地工作组: (4,4,3)，在 dispatch(4,4,3) 中设定
    vec3 lwNum = gl_NumWorkGroups;
    
    // 当前 [工作项] 在 {本地工作组} 中的位置: (1,2,1)
    vec3 ltId = gl_LocalInvocationID;
    
    // 当前 {本地工作组} 在 {全局工作组} 中的位置: (1,2,1)
    vec3 lwId = gl_WorkGroupID;
    
    // 当前 [工作项] 在 {全局工作组} 中的直接位置: (0,0,0)
    vec3 gtId = gl_GlobalInvocationID;
    
    // gl_LocalInvocationID 的一维扁平化形式
    uint k = gl_LocalInvocationIndex;
    
    // {全局工作组}每个维度上的基于 [工作项] (以其为计数单位)的尺寸
    vec3 gtNum = lwSize * lwNum;
    

    
    /*
    // 实例缓存索引 i 必须跟工作项索引的范围一致
    uint i = gl_LocalInvocationIndex;
    // Prevent going over the end of the buffer
    if (i < instanceCount)
    {
        // 用当前工作项在本地工作组中的索引位置作为位置坐标，可以绘制出一个本地工作组
        instPos[i].position = vec4(gl_LocalInvocationID*factor, 1.0);    
    }
    */
    
    // 绘制一个全局工作组中的所有工作项，闪烁，说明存在不一致的地方
    // 每个全局工作组中有多少个工作项
    // 共有6种索引值算法，相当于6种寻址方式，选择不同方向做段寄存器和偏移量
    //uint i = (gl_GlobalInvocationID.x+1) * (gl_GlobalInvocationID.y+1) * (gl_GlobalInvocationID.z+1);
    //先填满 (xoy) 平面
    //uint i = uint(gtId.z * gtNum.x * gtNum.y + gtId.y * gtNum.x + gtId.x);
    //uint i = uint(gtId.z * gtNum.x * gtNum.y + gtId.x * gtNum.y + gtId.y);
    //先填满 (xoz) 平面
    //uint i = uint(gtId.y * gtNum.x * gtNum.z + gtId.x * gtNum.z + gtId.z);
    //uint i = uint(gtId.y * gtNum.x * gtNum.z + gtId.z * gtNum.x + gtId.x);
    //先填满 (yoz) 平面
    //uint i = uint(gtId.x * gtNum.y * gtNum.z + gtId.y * gtNum.z + gtId.z);
    uint i = uint(gtId.x * gtNum.y * gtNum.z + gtId.z * gtNum.y + gtId.y);

    if (i < instanceCount)
    {
        // 按照 {全局工作组} 各个方向上的尺寸排列每个 {工作项}
        instPos[i].position = vec4(gl_GlobalInvocationID*factor, 1.0);  
    
        // 全局工作组尺寸用来计算颜色范围
        float colRange = uint(lwNum.x + lwNum.y + lwNum.z)/3;
        
        //当前工作项所在的本地工作组的id
        //同一个本地工作组中的所有工作项的这个参数向量都一样，可以用这个值构造同一个本地工作组的颜色
        vec4 col = vec4(lwId, colRange);
    
        // 需要求得当前工作项所在的本地工作组的序号
        instCol[i].color0 = normalize(col) * normalize(vec4(colR, 1.0));
        //instCol[i].color0 = normalize(col);
    }
}                    
]]
)


VisualWorkGroup = shader.compute(
[[
#version 430
#include <codea/common.glsl>

uniform uint instanceCount;
uniform uint ulx;

// 
struct Instance
{
vec4 position;
};

/*
layout(std430, binding = BUFFER0 ) buffer  Pos
{
    Instance instances[ ]; 
}; 
*/
// 定义 RW 型的 buffer，变量名为 instances，单元类型为结构体 Instance，使用 BUFFER0 缓存位置 
BUFFER_RW(instances, Instance, BUFFER0);


//输入布局限定符声明一个 16x8x1 的本地工作组----gl_WorkGroupSize
//layout( local_size_x = 1, local_size_y = 1, local_size_z = 1 ) in;
//NUM_THREADS(1, 1, 1)
NUM_THREADS(2, 2, 2)

void main()     
{
uint factor = 5;

// 内置变量，只能在 compute shader 中使用
// 本地工作组尺寸：每个维度上有多少个工作项 (local_size_x, local_size_y, local_size_z)
vec3 lwSize = gl_WorkGroupSize;

// 全局工作组尺寸：每个全局工作组中有多少个本地工作组: (4,4,3)，在 dispatch(4,4,3) 中设定
vec3 lwNum = gl_NumWorkGroups;

// 当前工作项在本地工作组中的位置: (1,2,1)
vec3 ltId = gl_LocalInvocationID;

// 当前本地工作组在全局工作组中的位置: (1,2,1)
vec3 lwId = gl_WorkGroupID;

// 当前工作项在全局工作组中的直接位置: (0,0,0)
vec3 gtId = gl_GlobalInvocationID;

// gl_LocalInvocationID 的一维扁平化形式
uint k = gl_LocalInvocationIndex;

// 全局工作组每个维度上的基于工作项(以其为计数单位)的尺寸
vec3 gtNum = lwSize * lwNum;

// 利用内置变量实例ID: gl_GlobalInvocationID 作为 buffer 索引
// Get the current instance to work on

/*
// 实例缓存索引 i 必须跟工作项索引的范围一致
uint i = gl_LocalInvocationIndex;
// Prevent going over the end of the buffer
if (i < instanceCount)
{
    // 用当前工作项在本地工作组中的索引位置作为位置坐标，可以绘制出一个本地工作组
    instances[i].position = vec4(gl_LocalInvocationID*factor, 1.0);    
}
*/

// 绘制一个全局工作组中的所有工作项，闪烁，说明存在不一致的地方
// 每个全局工作组中有多少个工作项
//uint i = (gl_GlobalInvocationID.x+1) * (gl_GlobalInvocationID.y+1) * (gl_GlobalInvocationID.z+1);
//uint i = uint(gtId.z * gtNum.x * gtNum.y + gtId.y * gtNum.x + gtId.x);
//uint i = uint(gtId.z * gtNum.x * gtNum.y + gtId.x * gtNum.y + gtId.y);
uint i = uint(gtId.y * gtNum.x * gtNum.z + gtId.x * gtNum.z + gtId.z);
if (i < instanceCount)
{
    instances[i].position = vec4(gl_GlobalInvocationID*factor, 1.0);  
}
  

}                    
]]
)

